proc ListSelect { title parent trace_type trace_id list_choice title_choice list_picked title_picked } {
	global Process

	# Create the Window.
	if ![winfo exists $parent] { set parent "" }
	set w $parent.lstsel
	if [winfo exists $w] {
		wm deiconify $w
		return
	}
	toplevel $w
	wm title $w $title
    	wm transient $w [winfo toplevel [winfo parent $w]]

	set wf $w.f
	pack [frame $wf] -expand true -fill both

	# Create a Label for info.
	label $wf.title -text $title \
		-borderwidth 2 -relief ridge
	pack $wf.title -side top -pady 2m

	# Create two lists
	frame $wf.choice
	set wchoice [ScrolledListbox $wf.choice.listb \
		-width 20 -height 10 -setgrid true -cursor {plus}]
	frame $wf.picked
	set wpicked [ScrolledListbox $wf.picked.listb \
		-width 20 -height 10 -setgrid true -cursor {pirate}]

	# Insert Labels for info
	label $wf.choice.title -text $title_choice \
		-borderwidth 2 -relief ridge
	label $wf.picked.title -text $title_picked \
		-borderwidth 2 -relief ridge
	pack $wf.picked.title $wf.choice.title \
		-side bottom -pady 2m -fill both

	# pack the list windows side by side
	pack $wf.choice $wf.picked -side left \
		-expand true -fill both -pady 5m -padx 5m

	# Add a prompt (for a "manual" selection)
	pack [frame $wf.prompt] -side top -padx 2m -pady 2m
    	message $wf.prompt.msg -text "Files/Filters" -relief ridge -width 96
    	entry $wf.prompt.entry -width 12
    	pack $wf.prompt.msg $wf.prompt.entry
	bind $wf.prompt.entry <Return>  \
		[ list ListSelect_Prompt %W $wchoice $wpicked ]

	# Create Buttons
    	pack [frame $wf.button] -side top
    	pack [ button $wf.button.reset -text "Reset" -cursor {exchange}\
		-command [list ListSelect_Reset $wpicked $list_picked] ] \
            -side top -fill x
    	pack [ button $wf.button.sel_all -text "All" \
		-command [list ListSelect_All $wchoice $wpicked] ] \
            -side top -fill x
    	pack [button $wf.button.unsel_all -text "None" -cursor {pirate} \
		-command [list ListSelect_None $wpicked] ] \
            -side top -fill x
    	button $wf.button.done -text "Done" -cursor {heart} \
		-command [list ListSelect_Done $w $wpicked $trace_type $trace_id ]
    	pack $wf.button.done -side top -fill x
    	pack [button $wf.button.cancel -text "Cancel" \
		-command [list ListSelect_Cancel $w $trace_type $trace_id $list_choice] ] \
            -side top -fill x

	center_window $w

	
	# Selecting in choice moves items into picked
	bind $wchoice <ButtonPress-1> \
		{ListSelect_Start %W %y}
	bind $wchoice <B1-Motion> \
		{ListSelect_Extend %W %y}
	bind $wchoice <ButtonRelease-1> \
		[list ListSelect_End %W %y $wpicked]

	# Selecting in picked deletes items
	bind $wpicked <ButtonPress-1> \
		{ListSelect_Start %W %y}
	bind $wpicked <B1-Motion> \
		{ListSelect_Extend %W %y}
	bind $wpicked <ButtonRelease-1> \
		{ListSelect_DeleteEnd %W %y}

	# Insert all the choices and pre-picked items
	ListSelect_Reset $wchoice $list_choice
	ListSelect_Reset $wpicked $list_picked

	# Ok, status is : running
	set Process($trace_type,selectfiles,$trace_id,status) running

}

proc ListSelect_None {w} {
	$w delete 0 end
}

proc ListSelect_All {wchoice wpicked} {
	foreach item [$wchoice get 0 end] {
		$wpicked insert end $item
	}

	ListSelect_Reset $wpicked [RemoveDuplicateItemsInList [$wpicked get 0 end]]
}

proc ListSelect_Reset {w items} {
	$w delete 0 end
	eval {$w insert 0} $items
}

proc ListSelect_Prompt {wprompt wchoice wpicked} {
	# get the prompt items (from an entry widget !)
	set list(prompt) [$wprompt get]
	# get the picked items (from a listbox widget !)
	set list(picked) [$wpicked get 0 end]
	# get the choice items (from a listbox widget !)
	set list(choice) [$wchoice get 0 end]

	# generate the new picked items
#	set list_out_new [lsort -ascii [AppendInList $list(picked) [ExtractInList $list(choice) $list(prompt)]]]
	set list_out_new [AppendInList $list(picked) [ExtractInList $list(choice) $list(prompt)]]

	# update the picked window with the new picked items
	$wpicked delete 0 end
	foreach name $list_out_new { $wpicked insert end $name }
}

proc ListSelect_Done {w wpicked trace_type trace_id} {
# End of selection
	global Process

	# Get the picked items as an ascii ordered list
#	set list [lsort -ascii [$wpicked get 0 end]]
	set list [$wpicked get 0 end]

	# end
	set Process($trace_type,selectfiles,$trace_id,value) $list
	set Process($trace_type,selectfiles,$trace_id,status) done
	catch {destroy $w}
}
proc ListSelect_Cancel {w trace_type trace_id list_choice} {
# Cancel , return original list of choices
	global Process

	set Process($trace_type,selectfiles,$trace_id,value) $list_choice
	set Process($trace_type,selectfiles,$trace_id,status) cancel
	catch {destroy $w}
}

proc ListSelect_Start { w y } {
	$w select anchor [$w nearest $y]
}

proc ListSelect_Extend { w y } {
	$w select set anchor [$w nearest $y]
}

proc ListSelect_End {wchoice y wpicked} {
	$wchoice select set anchor [$wchoice nearest $y]
	foreach i [$wchoice curselection] {
		$wpicked insert end [$wchoice get $i]
	}
	ListSelect_Reset $wpicked [RemoveDuplicateItemsInList [$wpicked get 0 end]]
	}

proc ListSelect_DeleteEnd {w y} {
	$w select set anchor [$w nearest $y]
	foreach i [lsort -decreasing [$w curselection]] {
		$w delete $i
	}
}
